<?php
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

require_once 'shared.inc.php';


class PageFormEditNode extends PageFormEdit
{
	public $iOrder, $idDeleteLink;
	protected $aParent;
	
	
	protected function getParameters() { return concatLists(parent::getParameters(), 'iOrder'); }
	
	protected function getCurrentNodeID() { return ($this->obj) ? $this->obj->id : 0; }
	
	
	protected function initialize()
	{
		$_REQUEST->prepare('bLeftPane');
		$this->loadRequestParams('idDeleteLink');
		self::initializeGeneric(node::$table->sName);
	}
	
	
	/// Process path part changing (node::sPathPart).
	protected function beforeSubmit(array &$aField)
	{
		if (isset($aField['sPathPart'])) {
			$sPathPart = $aField['sPathPart'];
			
			$obj = $this->obj;
			
			if ($this->iOrder < nodeLast && !$sPathPart)
				$sError = 'Veuillez spcifier un nom de dossier.';
			else{
				$aBrotherPathPart = sql::selectField(node::$table->sName, 'sPathPart',
					sql::where($obj->getBrotherSqlCondition()) . ' AND id <> ' . $obj->id);
				
				$sError = $obj->onPathPartChanged($sPathPart, $this->aParent, $aBrotherPathPart);
			}
			
			if ($sError)
				$this->form->error($sError);
		}
	}
	
	
	protected function submit(array $aField)
	{
		sql::update(node::$table->sName, $aField, sql::whereId($this->id));
		return 0;
	}
	
	
	protected function cancel()
	{
		if ($this->sessionManager->hasRightDeleteCreating($this->obj) && $this->obj instanceof node) {
			sql::delete(node::$table->sName,
				sql::where(array(
					'id'     => $this->id,
					'iFinal' => finalTemp,
				)));
			if (sql::getAffectedCount() > 0)
				sql::delete('link', sql::whereOne('idNode', $this->id));
		}
	}
	
	
	protected function contents()
	{
		$obj = $this->obj;
		$info = $this->info;
		$sTable = node::$table->sName;
		
		if ($this->idDeleteLink)
			sql::delete('link', sql::where(array(
				'id'     => $this->idDeleteLink,
				'idNode' => $this->id,
			)));
		
		// Process property actions:
		// - modify existing links: action with > 0 ID (the link ID)
		// - add new links: action with < 0 ID
		$sRedirectURL = null;
		if ($this->bSubmit) {
			foreach ($info->aProperty as $iProperty => $nbAdd) {
				$sURL = $this->processPropertyActions($iProperty);
				if ($sURL)
					$sRedirectURL = $sURL;
			}
		}
		
		if ($this->bCreating) {
			$obj->iNode    = $this->iObject;
			$obj->idParent = $this->idParent;
			$sWhereBrother = sql::where($obj->getBrotherSqlCondition());
			
			sql::lockTables(array($sTable));
			
			// Maximum value for iOrder field of the inserted node
			$iOrderBound = sql::selectOneField($sTable,
				'MAX(iOrder)+1', $sWhereBrother);
			
			$iOrder = ($this->iOrder < 0)
				? $iOrderBound
				: min($this->iOrder, $iOrderBound);
			
			if ($iOrder < $iOrderBound)
				sql::update($sTable, array('!iOrder' => 'iOrder+1'),
					$sWhereBrother . ' AND iOrder>=' . $iOrder);
			
			$obj->iOrder     = $iOrder;
			$obj->dtCreation = time();
			$obj->iFinal     = finalTemp;
			$obj->id         =
			$this->id        = sql::insertIntoObj($sTable, $obj,
				getList('iNode;idParent;iOrder;sTitle;sDetails;sPathPart;sURL;dtCreation;iFinal'));
			
			// Get the default values from the SQL table
			$objNew = sql::selectOneFetch($sTable, array('*'), sql::whereId($this->id), $sTable);
			foreach ($objNew as $sField => $value) {
				$valueOld = $obj->$sField;
				if (!is_array($valueOld) && !is_object($valueOld))
					$obj->$sField = $value;
			}
			
			sql::unlockTables();
		}
		
		$sIntroText = '';
		if ($this->aParent) {
			$aAncestor = $this->aParent;
			if (!$this->bCreating)
				$aAncestor[] = $obj;
			$aLinkAncestor = array();
			foreach ($aAncestor as $objAncestor)
				$aLinkAncestor[] = $objAncestor->link($this);
			$sIntroText .= html::div($this->linkBar('Chemin', $aLinkAncestor), attr::className('parents'));
		}
		
		$s = self::contentsGeneric($sIntroText);
		
		if ($sRedirectURL)
			self::redirect($sRedirectURL);
		
		return $s;
	}
	
	
	/// @brief Process the fields created by FormObject::pushSetLinks() and FormObject::pushAddLinks().
	/// Return an URL if we should redirect to another page, \c null if no redirection is needed.
	private function processPropertyActions($iProperty)
	{
		$info = ObjectTypeInfo::get($iProperty);
		$sTable = node::$table->sName;
		
		$aid = $_REQUEST->getSafe(FormObject::formatAction($info->sName));
		if ($aid) {
			// Process FormObject::pushAddLinks() fields: an aid... array,
			// strictly positive indexes refer to existing links,
			// strictly negative indexes refer to links to be created.
			
			$aSubProperty = $_REQUEST->getSafe(FormObject::formatAction($info->sName . 'Sub'));
			
			$sPropertyTable = $info->table->sName;
			sql::lockTables(array('link', $sTable, $sPropertyTable));
			
			foreach ($aid as $iIndex => $idProperty) {
				if ($idProperty <= 0)
					continue;
				
				settype($aSubProperty[$iIndex], 'integer');
				
				$aUpdate = array(
					'idProperty' => $idProperty,
				);
				$aCriteria = array(
					'idNode'    => $this->id,
					'iProperty' => $iProperty,
				);
				$aInsertInto = $aUpdate + $aCriteria;
				
				if ($iIndex > 0)
					sql::update(array('link', $sTable), $aUpdate,
						sql::where($aCriteria + array(
							'link.id' => $iIndex,
							$sTable . '.id' => $idProperty,
					)));
				else if (1 == sql::count($sPropertyTable, sql::whereId($idProperty)))
					sql::replace('link', $aInsertInto);
			}
			
		}else{
			// Process FormObject::pushSetLinks():
			// a s... field containing the links list
			
			$aLinkOld = projectTo($this->obj->getPropertiesLink($iProperty), 'idProperty');
			
			// Associative array (name => ID)
			$aPropertyByName = array();
			foreach (getProperties($iProperty) as $property)
				$aPropertyByName[strtolower(trim($property->getLinkTitle()))] = $property->id;
			
			// Split the properties list by newline
			$aLinkNewName = preg_split('/[\\n\\r]+/', $_REQUEST->getSafe('s' . $info->sName),
				-1, PREG_SPLIT_NO_EMPTY);
			
			$aLinkNew = array();
			foreach ($aLinkNewName as $sName) {
				$sNameClean = trim($sName);
				$sName = strtolower($sNameClean);
				if (!isset($aPropertyByName[$sName])) {
					$this->form->error('lment introuvable&nbsp;: ' . html::kbdText($sNameClean));
					return;
				}
				$aLinkNew[] = $aPropertyByName[$sName];
			}
			makeUnique($aLinkNew);
			
			$aDelete = array_diff($aLinkOld, $aLinkNew);
			$aAdd    = array_diff($aLinkNew, $aLinkOld);
			
			sql::lockTables(array('link'));
			
			if ($aDelete)
				sql::delete('link', sql::whereOne('idNode', $this->id) . ' AND ' . sql::formatInIntList('idProperty', $aDelete));
			
			$aInsertInto = array(
				'idNode'     => $this->id,
				'iProperty'  => $iProperty,
			);
			foreach ($aAdd as $idProperty) {
				$aInsertInto['idProperty'] = $idProperty;
				sql::replace('link', $aInsertInto);
			}
		}
		
		sql::unlockTables();
		
		if (!$_REQUEST->getSafe('bSubmitCreateProperty_' . $iProperty))
			return null;
		
		$sReturnURL = $this->getSelfTarget(targetParams)->url();
		$aParam = array(
			'iObject'        => $iProperty,
			'idParent'       => $this->id,
			'sExitURL'       => $sReturnURL,
			'sDeleteURL'     => $sReturnURL,
			'sURLFieldNewId' => FormObject::formatAction($info->sName, -1),
		);
		$target = new target('edit_property.php', $aParam);
		$this->targetAddSessionParams($target);
		return $target->url();
	}
	
	
	/// Push the form fields of a node.
	public function pushFormNode($form, $sLabelURL, $sCommentURL)
	{
		$info = $this->info;
		$obj  = $this->obj;
		
		$form->pushText('sTitle', maxLengthTitle, -1, 'Nom', 'Titre complet ' . $info->sDuDe);
		$form->pushTextArea('sDetails', 'Dtails', 3, 'Description dtaille ' . $info->sDuDe);
		$form->pushText('sURL', maxLengthURL, -1, $sLabelURL, $sCommentURL, 'http://');
		
		if ($this->isAdmin()) {
			
			// Flags editing
			foreach ($obj->parent->getNodeFlags() as $nodeFlag) {
				$sPHPName = $nodeFlag->sPHPName;
				$iValue = $form->getValue($sPHPName);
				$form->pushElementLabel($sPHPName, $nodeFlag->sName,
					form::select('ID' . $sPHPName, $nodeFlag->aValue, $iValue), $nodeFlag->sDescription);
			}
			
			$sPathParent = html::kbdText(path::appendSeparator(concatPathPart($this->aParent)));
			$bFile = ($obj->iNode == nodeLast);
			
			$sComment = ($bFile)
				? 'Le fichier de ' . $sPathParent
				: (($this->aParent)
					? 'Le sous-dossier de ' . $sPathParent
					: 'Le dossier');
			$sComment .= ' qui contient les lments ' . $info->sDuDe;
			$form->pushText('sPathPart', maxLengthURL, -1, ($bFile) ? 'Fichier' : 'Dossier', $sComment);
		}
		
		foreach ($info->aProperty as $iProperty => $nbAdd) {
			$form->pushSeparator();
			if ($nbAdd == 0)
				continue;
			
			if ($nbAdd < 0)
				$form->pushSetLinks($iProperty);
			else /* ($nbAdd > 0)*/ {
				$nbPresent = $form->pushEditLinks($iProperty);
				$form->pushAddLinks($iProperty, max(1, $nbAdd - $nbPresent));
			}
			$form->pushCreateProperty($iProperty);
		}
	}
	
	
	public function getFormResource(FormObject $form)
	{
		$this->pushFormNode($form, 'Page Web', '');
		
		return $this->getSubmitButtons();
	}
	
	
	public function getFormSession(FormObject $form)
	{
		$this->pushFormNode($form, 'Page Web', '');
		
		return $this->getSubmitButtons();
	}
	
	
	public function getFormDocGroup(FormObject $form)
	{
		$this->pushFormNode($form, 'Page Web', 'Page commune  tous les documents du groupe');
		
		return $this->getSubmitButtons('Valider puis diter les documents');
	}
	
	
	public function getFormDocument(FormObject $form)
	{
		$this->pushFormNode($form, 'URL', 'Adresse Internet du document');
		
		return array();
	}
	
	
	private function getSubmitButtons($sCreateChildTitle = null)
	{
		if ($sCreateChildTitle === null) {
			$infoSub = ObjectTypeInfo::get($this->obj->iNode + 1);
			$sCreateChildTitle = 'Valider puis crer ' . $infoSub->sUnUne;
		}
		
		return array(
			form::submit('bSubmitCreateChild', $sCreateChildTitle),
			form::submit('bSubmitView', 'Valider et visualiser'),
		);
	}
}

Page::output(new PageFormEditNode);

?>