// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

var JScriptHeaderLoaded;

if (!JScriptHeaderLoaded) {

JScriptHeaderLoaded = true;

Math.minmax = function(value, min, max) {
	return Math.max(min, Math.min(value, max));
}

String.prototype.lastChar = function(index) {
	return (this.length > 0) ? this.charAt(this.length - 1) : '';
}

String.prototype.escapeHTML = function() {
	return this.replace("&", "&amp;").replace("<", "&lt").replace(">", "&gt");
}

String.prototype.escape = function() {
	return escape(this);
}

String.prototype.trim = function() {
	return this.match(/^\s*((.|\s)*\S|)\s*$/)[1];
}

String.prototype.wordAdd = function(word) {
	return ((" " + this).replace(" " + word, "") + " " + word).trim();
}

String.prototype.wordRemove = function(word) {
	return ((" " + this).replace(" " + word, "")).trim();
}

String.prototype.wordSet = function(word, present) {
	var s = (" " + this).replace(" " + word, "");
	if (present)
		s += " " + word;
	return s.trim();
}

String.prototype.rot13 = function() {
	var codea = 'a'.charCodeAt(0), codez = codea + 25;
	var codeA = 'A'.charCodeAt(0), codeZ = codeA + 25;
	var s = "";
	for (var i = 0; i < this.length; i++) {
		var c = this.charCodeAt(i);
		if (codea <= c && c <= codez)
			c = codea + ((c - codea + 13) % 26);
		else if (codeA <= c && c <= codeZ)
			c = codeA + ((c - codeA + 13) % 26);
		s += String.fromCharCode(c);
	}
	return s;
}


function enableJsExtensions() {

window.enableJsExtensions = function () {};

Array.prototype.indexOf = function(element) {
	for (var i in this)
		if (this[i] === element)
			return i;
	return null;
}

Array.prototype.remove = function(element) {
	for (;;) {
		var i = this.indexOf(element);
		if (i === null)
			break;
		this.splice(i, 1);
	}
}

Array.prototype.removeAt = function(index) {
	this.splice(index, 1);
}

Array.prototype.toURLArgs = function(name) {
	name += '[';
	var args = [];
	var index = 0;
	for (var i in this)
		if (typeof this[i] != "function")
			args[name + (index++) + ']'] = this[i];
	
	return args;
}

Array.prototype.filter = function() {
	var copy = [];
	for (var i in this) {
		var value = this[i];
		if (this.hasOwnProperty(i) && value)
			copy.push(value);
	}
	return copy;
}

Array.prototype.toJSONString = Array_toJSONString;


Object.prototype.escapeURLArgs = function() {
	var urlArgs = "";
	for (var name in this)
		if (typeof this[name] != "function") {
			if (urlArgs.length > 0)
				urlArgs += '&';
			urlArgs += name + '=' + this[name].escapeURLArg();
		}
	return urlArgs;
}

Object.prototype.escapeURLArg = function() {
	return encodeURIComponent((typeof this == "boolean")
		? toint(this).toString()
		: this.toString());
}


Object.prototype.iter = function(f) {
	for (var i in this)
		if (typeof this[i] != "function")
			f(this[i]);
}

Object.prototype.iteri = function(f) {
	for (var i in this)
		if (typeof this[i] != "function")
			f(i, this[i]);
}


Object.prototype.map = function(f) {
	var mapped = [];
	for (var i in this)
		if (typeof this[i] != "function")
			mapped[i] = f(this[i]);
	return mapped;
}

Object.prototype.mapi = function(f) {
	var mapped = [];
	for (var i in this)
		if (typeof this[i] != "function")
			mapped[i] = f(i, this[i]);
	return mapped;
}


Object.prototype.itert = function(f) {
	for (var i = 0; i < this.length; i++)
		f(this[i]);
}

Object.prototype.iterti = function(f) {
	for (var i = 0; i < this.length; i++)
		f(i, this[i]);
}

Object.prototype.toJSONString = Object_toJSONString;

}


function itert(tab, f) {
	for (var i = 0; i < tab.length; i++)
		f(tab[i]);
}

function iterti(tab, f) {
	for (var i = 0; i < tab.length; i++)
		f(i, tab[i]);
}


// AJAX

function ajaxSend(target, params, eventCallback) {
	var xhr = null;
	try{
		if (window.XMLHttpRequest)
			xhr = new XMLHttpRequest();
		else if (window.ActiveXObject) {
			try{
				xhr = new ActiveXObject("Msxml2.XMLHTTP");
			}catch(e) {
				xhr = new ActiveXObject("Microsoft.XMLHTTP");
			}
		}
	}catch(e) {}
	
	xhr.onreadystatechange = function() {
		if (xhr.readyState == 4)
			eventCallback(xhr);
	}
	
	xhr.open("POST", target, true);
	xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
	xhr.send(params.escapeURLArgs());
	return true;
}


// The callback takes as arguments the returned XML root object
// and the XMLHttpRequest object
function ajax(target, params, callback) {
	return ajaxSend(target, params, function(xhr) {
		callback(xhr.responseXML.documentElement, xhr);
	});
}

function ajaxLocal(params, callback) {
	return ajax("?", params, callback);
}


function ajaxJSON(target, params, action, data, callback) {
	if (action !== null)
		params[action] = data.toJSONString();
	
	return ajaxSend(target, params, function(xhr) {
		callback(xhr.responseText.parseJSON(), xhr);
	});
}

function ajaxJSONLocal(params, action, data, callback) {
	return ajaxJSON("?", params, action, data, callback);
}


// XML manipulation

function xmltag(node, tagname) {
	var elem = xmlnode(node, tagname);
	if (elem == null)
		return null;
	return (elem.textContent != null) ? elem.textContent : elem.text;
}

function xmlnode(node, tagname) {
	return node.getElementsByTagName(tagname)[0];
}


// DOM manipulation

function id(sid) {
	return document.getElementById(sid);
}

function loadByID(sids) {
	sids.split(";").iter(function(sid) {
		window[sid] = id(sid);
	});
}

function classAdd(elem, className) {
	elem.className = ((" " + elem.className).replace(" " + className, "") + " " + className).trim();
}

function classRemove(elem, className) {
	elem.className = ((" " + elem.className).replace(" " + className, "")).trim();
}

function classSet(elem, className, present) {
	var classNames = (" " + elem.className).replace(" " + className, "");
	if (present)
		classNames += " " + className;
	elem.className = classNames.trim();
}

function elemCreate(type, className, text) {
	var elem = document.createElement(type);
	if (className)
		elem.className = className;
	if (text)
		setxt(elem, text);
	return elem;
}

function elemRemove(elem) {
	if (elem && elem.parentNode)
		elem.parentNode.removeChild(elem);
}

function elemAncestor(elem, level) {
	for (; elem && level > 0; level--)
		elem = elem.parentNode;
	return elem;
}

function elemHover(elem, classHover, classNormal) {
	if (classHover == null)
		classHover = "hover";
	classHover = " " + classHover;
	if (classNormal != null)
		classNormal = " " + classNormal;
	
	elem.onmouseover = function() {
		var className = elem.className;
		if (classNormal != null)
			className = (" " + className).replace(classNormal, "");
		elem.className = (className + classHover).trim();
	};
	elem.onmouseout = function() {
		var className = (" " + elem.className).replace(classHover, "");
		if (classNormal != null)
			className += classNormal;
		elem.className = className.trim();
	};
}

function getxt(elem) {
	return (elem.textContent != null) ? elem.textContent : elem.innerText;
}

function setxt(elem, text) {
	elem.textContent = elem.innerText = (text == null) ? "" : text;
}

function txtSelectAll(elem, giveFocus) {
	if (elem.createTextRange) {
		var sel = elem.createTextRange();
		sel.moveEnd("textedit");
		sel.select();
	}else
		elem.setSelectionRange(0, elem.value.length);
	if (giveFocus)
		elem.focus();
}

function isVisible(elem) {
	return elem.style.visibility != "hidden";
}

function setVisible(elem, visible) {
	if (visible < 0)
		visible = "hidden";
	else if (visible > 0)
		visible = "visible";
	else
		visible = "inherit";
	elem.style.visibility = visible;
}

function isDisplayVisible(elem) {
	return elem.style.display != "none";
}

function setDisplay(elem, visible) {
	elem.style.display = (visible) ? "" : "none";
}

function setDisplayBlock(elem, visible) {
	elem.style.display = (visible) ? "block" : "none";
}

function setDisplayInline(elem, visible) {
	elem.style.display = (visible) ? "inline" : "none";
}

function setPos(elem, x,y) {
	elem.style.left = x + "px";
	elem.style.top  = y + "px";
}

function getAbsPos(elem) {
	var pos = new Point(0, 0);
	for (; elem != null; elem = elem.offsetParent) {
		pos.x += elem.offsetLeft;
		pos.y += elem.offsetTop;
	}
	return pos;
}


function iframeDoc(iframe) {
	var doc = iframe.contentWindow || iframe.contentWindow;
	if (doc && doc.document)
		doc = doc.document;
	return doc;
}

function insertAfter(parent, elem, child) {
	if (child) {
		var childSucc = child.nextSibling;
		if (childSucc)
			parent.insertBefore(elem, childSucc);
		else
			parent.appendChild(elem);
	}else
		parent.insertBefore(elem, null);
}


// Debugging

function debug(value) {
	alert(getDebug(value));
}

function getDebug(value) {
	if (typeof value == "string")
		return value;
	var s = "";
	for (var field in value)
		if (value[field] != null)
			s += field + ": " + ((typeof value[field] != "function") ? value[field] : "<function>") + "\n";
	if (s == "")
		s = value;
	return s;
}

function debugAll(value) {
	alert(getDebugAll(value));
}

function getDebugAll(value) {
	var s = "";
	for (var field in value)
		if (value[field] != null)
			s += field + ": " + value[field] + " -- ";
	if (s == "")
		s = value;
	return s;
}

function dbgPrint(text) {
	document.body.appendChild(elemCreate("pre", null, text));
}

function dbgDump(value) {
	dbgPrint(getDebug(value));
}

function dbgDumpAll(value) {
	dbgPrint(getDebugAll(value));
}

function dbgerr(f) {
	try{
		f();
	}catch(e) {
		debug(e);
	}
}


// Paths manipulation

function dirname(path) {
	if (path.length > 0 && path.substr(path.length-1) == '/')
		path = path.substr(0, path.length-1);
	return path.substr(0, path.lastIndexOf('/'));
}

function normalizepath(path) {
	path = path.replace(/\\\\/g, '/');
	while (path.indexOf("//") >= 0)
		path = path.replace(/\/\//g, '/');
	while (path.lastChar() == '/')
		path = path.substr(0, path.length - 1);
	return path;
}


// Type casting, custom types

function tobool(x) {
	return (x && x != 0 && x != "0" && x != "");
}

function toint(b) {
	return (b) ? 1 : 0;
}

function Point(x, y) {
	this.x = x;
	this.y = y;
}


// Events

function eventSetup(e) {
	e = e || event;
	if (e != null) {
		e.stop = function() {
			if (this.stopPropagation)
				this.stopPropagation();
			else
				this.cancelBubble = true;
		}
		
		e.cancel = function() {
			this.stop();
			if (this.preventDefault)
				this.preventDefault();
			else
				this.returnValue = false;
			return false;
		}
		
		e.getSrc = function() {
			return (this.target != null) ? this.target : this.srcElement;
		};
		
		e.findSrc = function(tagName) {
			var src = this.getSrc();
			if (tagName != null) {
				tagName = tagName.toUpperCase();
				while (src != null && src.tagName != null && src.tagName.toUpperCase() != tagName)
					src = src.parentNode;
				if (src != null && src.tagName == null)
					src = null;
			}
			return src;
		};
		
		e.screenLocation = function() {
			return new Point(this.screenX, this.screenY);
		}
		
		e.offsetLocation = function() {
			return new Point(this.offsetX, this.offsetY);
		}
	}
	return e;
}

function eventAddInternal(elem, evt, f) {
	if (elem.addEventListener)
		elem.addEventListener(evt, f, false);
	else if (elem.attachEvent)
		elem.attachEvent("on" + evt, f);
}

function eventCatch(elem, evt, f, arg) {
	eventAddInternal(elem, evt, eventCatcher(f, arg));
}

function eventCatchSimple(elem, evt, f, arg) {
	eventAddInternal(elem, evt, function(e) {
		return f(arg);
	});
}

function eventSpy(elem, evt, f, arg) {
	eventAddInternal(elem, evt, function(e) {
		try{
			f(eventSetup(e), arg);
		}catch(e) {}
		return true;
	});
}

function catchLoad(f) {
	eventSpy(window, "load", function() {
		enableJsExtensions();
		f();
	});
}

function catchEscapeKey(elem, f, arg) {
	eventCatch(elem, "keydown", function(e) {
		if (e.keyCode == 27)
			f(arg);
		return true;
	});
}


function eventCanceler(e) {
	eventSetup(e).cancel();
}

function eventStopper(e) {
	eventSetup(e).stop();
}

function eventCatcher(f, arg) {
	return function(e) {
		return f(eventSetup(e), arg);
	};
}

function eventCall(f, e, arg) {
	return f(eventSetup(e), arg);
}

function functionCaller(f, arg) {
	return function() { return f(arg); }
}

function functionCallerReturn(f, arg, ret) {
	return function() { f(arg); return ret; }
}

function functionCallerFalse(f, arg) {
	return function() { f(arg); return false; }
}


// JSON library: convert JScript values from/to strings

Array_toJSONString = function() {
	var a = ['['], b, l = this.length, v;
	
	function p(s) {
		if (b)
			a.push(',');
		a.push(s);
		b = true;
	}
	
	for (var i = 0; i < l; i += 1) {
		v = this[i];
		switch (typeof v) {
			case 'undefined':
			case 'function':
			case 'unknown':
				break;
			case 'object':
				if (v) {
					if (typeof v.toJSONString === 'function')
						p(v.toJSONString());
				}else
					p("null");
				break;
			default:
				p(v.toJSONString());
		}
	}
	a.push(']');
	return a.join('');
};

Boolean.prototype.toJSONString = function() {
	return String(this);
};

Date.prototype.toJSONString = function() {
	function f(n) {
		return n < 10 ? '0' + n : n;
	}
	
	return '"' + this.getFullYear() + '-' +
		f(this.getMonth() + 1) + '-' +
		f(this.getDate()) + 'T' +
		f(this.getHours()) + ':' +
		f(this.getMinutes()) + ':' +
		f(this.getSeconds()) + '"';
};

Number.prototype.toJSONString = function() {
	return isFinite(this) ? String(this) : "null";
};

Object_toJSONString = function() {
	var a = ['{'], b, i, v;
	
	function p(s) {
		if (b)
			a.push(',');
		a.push(i.toJSONString(), ':', s);
		b = true;
	}
	
	for (i in this)
		if (this.hasOwnProperty(i)) {
			v = this[i];
			switch (typeof v) {
				case 'undefined':
				case 'function':
				case 'unknown':
					break;
				case 'object':
					if (v) {
						if (typeof v.toJSONString === 'function')
							p(v.toJSONString());
					}else
						p("null");
					break;
				default:
					p(v.toJSONString());
			}
		}
	
	a.push('}');
	return a.join('');
};


(function(s) {
	var m = {
		'\b': '\\b',
		'\t': '\\t',
		'\n': '\\n',
		'\f': '\\f',
		'\r': '\\r',
		'"' : '\\"',
		'\\': '\\\\'
	};
	
	s.parseJSON = function(hook) {
		try{
			return eval('(' + this + ')');
		}catch(e) {
			return null;
		}
	};
	
	s.toJSONString = function() {
		if (/["\\\x00-\x1f]/.test(this))
			return '"' + this.replace(/([\x00-\x1f\\"])/g, function(a, b) {
				var c = m[b];
				if (c)
					return c;
				c = b.charCodeAt();
				return '\\u00' +
					Math.floor(c / 16).toString(16) +
					(c % 16).toString(16);
			}) + '"';
		return '"' + this + '"';
	};
})(String.prototype);



// Templates system
// A template is a DOM element containing sub-elements having IDs.
//
// When the template is created, a pointer to each sub-element having an ID
// is put into a property of the template itself.
// For example, with the HTML tree:
// <div id="mytemplate">...<span id="subelem">...</span>...</div>
// the DOM element of 'mytemplate' contains a property named 'subelem'
// pointing to the SPAN element.
// Moreover, when a template is created, its DOM element is removed from the DOM tree.
//
// When a template is instantiated, its element and all children elements are cloned.
// A suffix is appended to all elements having an ID to avoid duplicated IDs.
// Then the cloned element is made visible (display: block).


// Add variables in 'root' pointing to all nodes from 'node'
// having an ID, recursively. Each variable takes the ID as name.
function collectIds(root, node) {
	if (node.id)
		root[node.id] = node;
	itert(node.childNodes, function(child) {
		collectIds(root, child);
	});
}

// Same as collectIds, plus appends a suffix to all IDs.
// The variables created by collectIds have the name
// without the suffix.
function renameIds(root, node, suffix) {
	if (node.id) {
		root[node.id] = node;
		node.id += suffix;
	}
	if (node.htmlFor)
		node.htmlFor += suffix;
	itert(node.childNodes, function(child) {
		renameIds(root, child, suffix);
	});
}


// Transform a node into a template:
// - collect its IDs
// - remove it from the DOM
function createTemplate(elem) {
	collectIds(elem, elem);
	elemRemove(elem);
}

// Clone a template and append a suffix to the ID of all its elements.
// Make the new instance visible ('block' display style).
// Save the suffix in a 'suffix' variable. Calls collectIds
// to save pointers to all subnodes having an ID.
function instantiateTemplate(template, suffix) {
	var instance = template.cloneNode(true);
	renameIds(instance, instance, suffix);
	instance.suffix = suffix;
	setDisplayBlock(instance, true);
	return instance;
}



// Auto-completion: highlight words taken from a text

function AutoCompletion(expr) {
	var words = expr.split(" ").filter();
	var nbWord = words.length;
	for (var j = 0; j < nbWord; j++)
		words[j] = words[j].toLowerCase();
	
	this.words = words;
	this.wordsHTML = words.map(function(word) { return word.escapeHTML().toLowerCase(); });
	
	// Positive matching = all words are found in the text.
	this.match = function(text) {
		for (var j = 0; j < nbWord; j++)
			if (text.toLowerCase().indexOf(words[j]) < 0)
				return false;
		return true;
	};
	
	// Take a string, return an HTML string.
	this.highlight = function(text) {
		text = text.escapeHTML();
		this.wordsHTML.iter(function(word) {
			var i = 0;
			var lenWord = word.length;
			for (;;) {
				i = text.toLowerCase().indexOf(word, i);
				if (i < 0)
					break;
				text = text.substr(0, i) +
					"<b>" + text.substr(i, lenWord) + "</b>" +
					text.substr(i + lenWord);
				i += 6 + lenWord;
			}
		});
		return text;
	};
}

function unobfuscateMail(elem) {
	setxt(elem, getxt(elem).rot13().replace(/-at-/g, '@').replace(/-dot-/g, '.'));
}

}
