<?php defined('KERNEL_INCLUDED') or die('kernel should be included first');
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder

//-----------------------------------------------------------------
/// @file
/// @brief Links building:
/// - #target: URL builder
/// - #link: link builder: target + label, tooltip, CSS class
//-----------------------------------------------------------------


//-----------------------------------------------------------------
/// @brief Data of a link target: URL path, parameters, anchor.
//-----------------------------------------------------------------

class target
{
	public $sPath, $aParam, $sAnchor;
	
	function target($sPath, array $aParam = array(), $sAnchor = '')
	{
		if (!$sAnchor) {
			$iAnchorPos = strrpos($sPath, '#');
			if ($iAnchorPos !== false) {
				$sAnchor = substr($sPath, $iAnchorPos + 1);
				$sPath = substr($sPath, 0, $iAnchorPos);
			}
		}
		
		$this->sPath   = $sPath;
		$this->aParam  = $aParam;
		$this->sAnchor = $sAnchor;
	}
	
	function addParam($sKey, $value)
	{
		if ($sKey[0] != 'a')
			forceType($value, $sKey);
		$this->aParam[$sKey] = $value;
	}
	
	function addParams(array $aParam)
	{
		if ($aParam)
			foreach ($aParam as $sKey => $value)
				$this->addParam($sKey, $value);
	}
	
	function contains($sKey)
	{
		return array_key_exists($sKey, $this->aParam);
	}
	
	function getParam($sKey, $defaultValue)
	{
		return defaultElement($this->aParam, $sKey, $defaultValue);
	}
	
	function isEmpty() { return !$this->sPath; }
	
	function removeParams($sParams)
	{
		foreach (getList($sParams) as $sParam)
			unset($this->aParam[$sParam]);
	}
	
	function removeParamsFromPrefix($sPrefix)
	{
		foreach ($this->aParam as $sKey => $value)
			if (hasPrefix($sKey, $sPrefix))
				unset($this->aParam[$sKey]);
	}
	
	function url()
	{
		$sURL = $this->sPath;
		
		$sArgs = http_build_query($this->aParam);
		if ($sArgs)
			$sURL .= '?' . $sArgs;
		
		if ($this->sAnchor)
			$sURL .= '#' . $this->sAnchor;
		
		return $sURL;
	}
	
	public static function fromURL($sURL)
	{
		$aPart = array();
		regexp::match('|^([^\\?\\#]*)(\\?[^\\#]*)?(\\#.*)?$|U', $sURL, $aPart);
		
		if (isset($aPart[2])) {
			parse_str(substr($aPart[2], 1), $aParam);
			stripMagicQuotesArray($aParam);
		}else
			$aParam = array();
		
		return new target($aPart[1], $aParam, (isset($aPart[3])) ? substr($aPart[3], 1) : '');
	}
	
	
	public function form($iMethod, $sContents, array $aAttr = array())
	{
		$sURL = $this->sPath;
		if ($this->sAnchor)
			$sURL .= '#' . $this->sAnchor;
		
		return form::formTo($sURL, $iMethod, $sContents, $aAttr, $this->aParam);
	}
}


//-----------------------------------------------------------------
/// @brief Data necessary to build an HTML link:
/// target URL, label, tooltip, CSS class
//-----------------------------------------------------------------

class link
{
	public $target, $sLabel, $sClassName, $sToolTip = '';
	
	
	public function __construct($target, $sLabel, $sClassName = '', $sToolTip = '')
	{
		if (is_string($target))
			$target = new target($target);
		else if (!($target instanceof target))
			debug::fatalError(__CLASS__ . ' constructor first argument must be string or target');
		
		$this->target     = $target;
		$this->sLabel     = $sLabel;
		$this->sClassName = $sClassName;
		$this->sToolTip   = $sToolTip;
	}
	
	public function addParam($key, $value)
	{
		$this->target->addParam($key, $value);
	}
	
	public function html(array $aAttr = array())
	{
		appendDefault($aAttr, attr::className($this->sClassName) + attr::title($this->sToolTip));
		
		return ($this->target->isEmpty())
			? html::span($this->sLabel, $aAttr)
			: html::link($this->target->url(), $this->sLabel, $aAttr);
	}
}

?>