<?php defined('KERNEL_INCLUDED') or die('kernel should be included first');
// This file is part of DENIF, distributed under license GPLv3.
// Copyright 2006 Guillaume Ryder, Arthur Charguraud

//-----------------------------------------------------------------
/// @file
/// @brief #PageFormEdit: object editing page base class
//-----------------------------------------------------------------


//-----------------------------------------------------------------
/// Object editing page base class
//-----------------------------------------------------------------

abstract class PageFormEdit extends PageDenif
{
	const sSubmitButtons = 'bSubmitReturn;bSubmitContinue;bSubmitView;bSubmitCreateChild';
	
	public $iObject, $id, $idParent;
	public $bSubmit, $bSubmitReturn, $bSubmitContinue;
	public $bSubmitView, $bSubmitCreateChild;
	public $sExitURL, $sDeleteURL;
	public $sURLFieldNewId;
	
	public $obj;
	public $info;
	public $targetExit;
	protected $bCreating;
	protected $aParent;
	
	protected $form;
	
	
	protected function getParameters() { return concatLists(parent::getParameters(), 'iObject;id;idParent;sExitURL;sDeleteURL;sURLFieldNewId'); }
	
	protected function getTitle()
	{
		return
			(($this->bCreating) ? 'Crer' : 'diter') . ' ' .
			(($this->info) ? $this->info->sUnUne : 'un objet');
	}
	
	
	protected function initializeGeneric($sObjectTypeRange)
	{
		parent::initialize();
		
		$this->loadParameters();
		$this->loadRequestParams(self::sSubmitButtons . ';bCancel');
		
		if (!$this->wantEditing(false))
			throw new PageException;
		
		$iObject = $this->iObject;
		
		$this->info = $info = ObjectTypeInfo::get($iObject);
		if (!$info || !isDefineInRange($iObject, $sObjectTypeRange))
			$this->fatalError('Type dobjet invalide.');
		
		$sTable = $info->table->sName;
		$obj = $this->obj = ($this->id)
			? sql::selectOneFetch($sTable, array('*'),
				sql::whereId($this->id), $sTable)
			: null;
		$this->bCreating = !$obj;
		
		$bParentsCreated = ($obj instanceof node);
		if ($bParentsCreated)
			$this->aParent = $obj->loadAllParents(true);
		
		// Get the edited object. If none is found (creating mode),
		// create a temporary object in the database
		if ($this->bCreating) {
			
			// If the object is a node, get the node in which it has to be created.
			$objParent = null;
			if (($obj instanceof node) && $this->idParent)
				$objParent = sql::selectOneFetch(node::$table->sName, 
					array('*'), sql::whereId($this->idParent));
			
			if (!$this->sessionManager->hasRightCreate($this->iObject, $objParent))
				$this->fatalError('Vous navez pas lautorisation de crer cet objet.');
			$obj = $this->obj = $info->instantiate();
			$obj->idParent = $this->idParent;
			
		}else{
			if (!$this->sessionManager->hasRightModify($obj))
				$this->fatalError('Vous navez pas lautorisation de modifier cet objet.');
		}
		
		if ($obj instanceof node) {
			if (!$bParentsCreated)
				$this->aParent = $obj->loadAllParents(true);
			
			if (!$obj->parent)
				$obj->parent = ($this->aParent) ? end($this->aParent) : node::getDefault();
		}
		
		$this->bSubmit = false;
		foreach (getList(self::sSubmitButtons) as $sName)
			$this->bSubmit |= $this->$sName;
		foreach ($info->aProperty as $iProperty => $nbAdd)
			$this->bSubmit |= $_REQUEST->getSafe('bSubmitCreateProperty_' . $iProperty);
		
		$this->targetExit = $this->targetFromURL($this->sExitURL);
		
		if ($this->sDeleteURL)
			$this->targetDelete = $this->targetFromURL($this->sDeleteURL);
		else if ($iObject == nodeResource)
			$this->targetDelete = $this->targetFromURL('index_themes.php');
		else
			$this->targetDelete = clone $this->targetExit;
		
		if ($this->bCancel) {
			$this->cancel();
			self::redirect($this->targetDelete->url());
		}
		
		$this->form = new FormObject($this, $obj);
	}
	
	
	protected function contentsGeneric($sIntroText = '')
	{
		$info = $this->info;
		
		$form = $this->form;
		$sMethod = 'getForm' . $info->sName;
		$aButtonMore = $this->$sMethod($form);
		if (!$aButtonMore)
			$aButtonMore = array();
		
		$form->pushSeparator();
		
		if (!$this->bCreating)
			$form->pushDelete();
		$form->pushSubmit($aButtonMore);
		
		$s = '';
		
		if ($this->bSubmit) {
			$aField = $form->getFields();
			$this->beforeSubmit($aField);
			
			if ($form->isError()) {
				$s .= $form->getErrors();
				
			}else{
				$idNew = $this->submit($aField);
				
				if ($idNew) {
					if ($this->sURLFieldNewId) {
						$this->targetExit->addParams(array(
							$this->sURLFieldNewId => $idNew,
							'bSubmitContinue'     => true,
						));
						$this->sExitURL = $this->targetExit->url();
					}
					
					if ($this->bSubmitContinue)
						self::redirect($this->getSelfTarget(targetParams, array('id' => $idNew))->url());
				}
				
				$targetRedirect = null;
				
				if ($this->bSubmitReturn)
					// Return: follow the exit link
					$targetRedirect = $this->targetExit;
				
				else if ($this->obj instanceof node) {
					$node = $this->obj;
					
					if ($this->bSubmitView)
						// View: go to the edited node
						$targetRedirect = $node->getLinkTarget();
						
					else if ($this->bSubmitCreateChild) {
						// Create child: go to create child form or to the AJAX documents interface
						
						$iNodeChild = $node->iNode + 1;
						if ($iNodeChild < nodeDocument)
							$targetRedirect = getLinkCreateChild($iNodeChild, $node->id,
								($iNodeChild == nodeSession) ? 0 : -1, $this->targetExit)->target;
						else if ($iNodeChild == nodeDocument)
							$targetRedirect = getLinkEditDocs($node, $this->targetExit)->target;
					}
				}
				
				if ($targetRedirect) {
					$this->targetAddSessionParams($targetRedirect);
					self::redirect($targetRedirect->url());
				}
			}
		}
		
		$s .= $this->putTitle($this->getTitle());
		
		$s .= $sIntroText;
		
		$s .= $this->getSelfTarget(targetParams)->form(
			form::METHOD_MULTIPART, $form->html());
		
		return $s;
	}
	
	
	/// @brief Called just before PageFormEdit::submit() is called,
	/// allowing to check the fields, and eventually raise errors or modify the fields.
	protected function beforeSubmit(array &$aField)
	{
	}
	
	/// Return the ID of the created object, if any
	protected abstract function submit(array $aField);
	
	protected abstract function cancel();
}

?>